'use strict';

// ** Constants
const DEFAULT_ERROR_CODE = 'ERROR';
const DEFAULT_ERROR_MSG = 'An error occurred';

// ** Dependencies
const util = require('util');
const callsite = require('callsite');
const functions = require('./functions');

/**
 * Check if a value is an error
 * @param err
 * @returns {boolean}
 */
function isError(err) {
    return err instanceof Error;
}

function createError(code, msg, data) {

    // Empty Error
    if (arguments.length === 0) {
        code = DEFAULT_ERROR_CODE;
        msg = DEFAULT_ERROR_MSG;
    }

    // Wrap an Error passed as first argument
    if (arguments.length === 1 && isError(arguments[0])) {
        code = arguments[0].code || DEFAULT_ERROR_CODE;
        msg = arguments[0].message || DEFAULT_ERROR_MSG;
        data = arguments[0].data;
    }

    // Wrap an Error passed as second argument
    if (arguments.length === 2 && isError(arguments[1])) {
        code = arguments[0] || DEFAULT_ERROR_CODE;
        msg = arguments[1].message || DEFAULT_ERROR_MSG;
        data = arguments[1].data;
    }

    // Build a JS error message
    let message = `[${code}]`;

    // Append msg to message
    if (msg)
        message += ' ' + msg;

    // Add data to message
    if (data)
        message += ' ' + JSON.stringify(data);

    // Return a native JS error
    const error = new Error(message);

    error.code = code;
    error.msg = msg || '';

    if (data)
        error.data = data;

    return error;
}

module.exports = createError;